c 
c     Description
c     
c     View multiple radial spectra at once, organized in a grid of
c     X_fine and Y_fine values.  Used for testing etalon parallelism.
c     Requires radial spectra prodcued by RING routine.
c
c
c     Authors
c
c     Ben Weiner, updated by David Rupke
c
c     History
c
c     06dec    BJW   created
c     07nov    DSNR  heavy modifications
c

c=============
c DEFINITIONS
c=============

      implicit none

      integer NMAXIM,maxzone
c     Maximum number of spectra
      parameter (NMAXIM=50)
c     Maximum number of radial bins in spectra.
      parameter (maxzone=5000)

      real wave0,xfine(NMAXIM),yfine(NMAXIM),zfine(NMAXIM)
      real xcopy(NMAXIM),ycopy(NMAXIM),xuniq(NMAXIM),yuniq(NMAXIM)
      real cont(NMAXIM),inty(NMAXIM),cwl(NMAXIM),sg(NMAXIM),sl(NMAXIM)
      real sigsum(NMAXIM),wavefmax(NMAXIM),fpeak(NMAXIM)
      real wave(NMAXIM,maxzone),flux(NMAXIM,maxzone)
      real ferr(NMAXIM,maxzone)
      real wavetmp(maxzone),fluxtmp(maxzone),ferrtmp(maxzone)
      real frms(maxzone),avg(maxzone)
      real wavefit(maxzone),fluxfit(maxzone),ferrfit(maxzone)
      real wmin,wmax,fmin,fmax,fmaxtmp,wusemin,wusemax
      real wfmax,wfmin,wavefmaxref,wavefmaxdiff
      real xmin,xmax,ymin,ymax
      real vplot(maxzone),plotpkf(2),plotpkw(2)
      real vgtfit(5),fiterr(5),dv(5),fitguess(2)
      real dvpx,dvpy,vpxl,vpxh,vpyl,vpyh,vpwidth,chisq,sgint,slint
      real dummy

      integer npts(maxzone)
      integer ixpanl(NMAXIM),iypanl(NMAXIM)
      integer iplotord(NMAXIM),ifilenum(NMAXIM)
      integer i,j,k,l,nfile,nz,npfit,firstplot,ilen,flag,dupli,indexc
      integer pgstat,nargs,dofit,dolist,dops,doshift,ios,nxuniq,nyuniq
      integer nimstart,nimend,zcoarse(NMAXIM)
      integer iord

      character*120 fname,oname,rnamearr(NMAXIM)
      character*80 arg,rnametmp,xlab,ylab,siglabel,psfile
      character*80 filter(NMAXIM),time(NMAXIM),date(NMAXIM)
      character*80 toplabel1,toplabel2
      character toplabel*50,cline*60

c     PGPLOT subroutine declaration.
      integer pgbeg

C========================
C Set initial parameters 
C========================

C     Read file names from input list?
      dolist=0
C     Do Voigt profile fit?
      dofit=0
C     Shift plots to match peaks?
      doshift=0
c     Width of Voigt profile fit, in wavelength units.
      vpwidth=20
      
C     Get command-line arguments.
      nargs = iargc()
      if (nargs .lt. 2) then
         write (*,'(3(a),/,2(a),4(/,a),a)') "Usage: ringplot",
     $        " (first file no., ccd prefix assumed)"
     $        ," (last file no.)","   OR  ringplot --in "
     $        ,"(name of file containing filenames)" ,"Optional flags:",
     $        "     --shift  : Shift peaks to common wavelength",
     $        "     --fit    : Add automatic Voigt profile fit",
     $        "     --fw [X] : Window for Voigt "
     $        ,"profile fit [default 40 wavelength units]"
         stop
      end if
      call getarg(1,arg)
      if (arg.eq.'--in') then
         dolist = 1
         call getarg(2,fname)
      else
         read (arg,*) nimstart
         call getarg(2,arg)
         read (arg,*) nimend
      end if
      if (nargs .gt. 2) then
         do i=3,nargs
            call getarg(i,arg)
            if (arg .eq. '--fit') then
               dofit = 1
            end if
            if (arg .eq. '--shift') then
               doshift = 1
            end if
            if (arg .eq. '--fw') then
               if (i .eq. nargs) then
                  write(*,'(3(a))') 'Must specify width '
     $                 ,'of Voigt profile fit after switch '
     $                 ,'(e.g., ringplot ... --fw 30).'
                  stop
               else
                  call getarg(i+1,arg)
                  read(unit=arg,fmt='(F3.0)') vpwidth
               end if
            end if
         end do
      end if            

C     Voigt profile fit order and initial widths
      iord = 5
      sgint = 0.2
      slint = 0.2


C==================
C Read rings files
C==================

      write (*,'(/,a)') "Reading in spectra produced by RING."
c
C     For an input file.
c
 90   if (dolist .eq. 1) then
         open(2,file=fname,status='old',iostat=ios)
         if (ios .ne. 0) then 
            write(*,'(2(a))')
     $           "Error opening file containing *.rings filenames: "
     $           ,fname
            stop
         end if
         nfile = 1
C     Get *.rings filenames
 91      read(2,'(a)',iostat=ios) rnametmp
         ilen = index(rnametmp,' ')-1
C     Check for end-of-file
         if (ios .ne. 0) then
            nfile = nfile - 1
            close(2)
            go to 100
         end if
c     Format i4.4 prints leading zeroes.
         write(rnamearr(nfile),'(a,".rings")') rnametmp(1:ilen)
C     Read *.rings files
         open(3,file=rnamearr(nfile),status='old',iostat=ios)
         if (ios .ne. 0) then
            write(*,'(2(a))') "Error opening file: ",rnamearr(nfile)
            stop
         end if
         read(3,*)
         read(3,*) wave0,xfine(nfile),yfine(nfile),zfine(nfile)
     $        ,zcoarse(nfile)
         read(3,*)
         read(3,*) date(nfile),time(nfile),filter(nfile),wavefmax(nfile)
     $        ,fpeak(nfile)
         read(3,*)
C     Get radial spectra
         nz = 1
 92      read(3,*,iostat=ios) dummy,wave(nfile,nz),dummy,flux(nfile,nz)
     $        ,dummy,ferr(nfile,nz)
C     Check for end-of-file, otherwise keep reading.
         if (ios .ne. 0) then
            nz = nz - 1
            close(3)
            go to 95
         end if
         nz = nz+1
C     Go to next radial bin
         go to 92
C     Go to next filename
 95      nfile=nfile+1
         go to 91
c         
C     For range of files.
c
      else
         nfile = nimend - nimstart + 1
         do i=1,nfile
            write(rnamearr(i),'("ccd",i4.4,".rings")') nimstart+i-1
            open(3,file=rnamearr(i),status='old',iostat=ios)
            if (ios .ne. 0) then
               write(*,'(a,a30)') "Error opening file: ",rnamearr(i)
               stop
            end if
            read(3,*)
            read(3,*) wave0,xfine(i),yfine(i),zfine(i),zcoarse(i)
            read(3,*)
            read(3,*) date(i),time(i),filter(i),wavefmax(i),fpeak(i)
            read(3,*)
C     Get radial spectrum
            nz = 1
 96         read(3,*,iostat=ios) dummy,wave(i,nz),dummy,flux(i,nz),dummy
     $           ,ferr(i,nz)
C     Check for end-of-file, otherwise keep reading.
            if (ios .ne. 0) then
               nz = nz - 1
               close(3)
               go to 97
            end if
            nz = nz+1
C     Go to next radial bin
            go to 96
 97         continue
         end do
      end if
            
 100  continue

C=================
C Sort rings data
C=================

C     Get wavelength range to use
      write(*,'(a)') "Plotting grid of radial spectra."
 105  write(*,'(a,$)')
     $     "Choose minimum and maximum of plot region [use all]:"
      read (*,'(a)') cline
      if (cline(1:4) .eq. '    ') then
         wusemin = -1
         wusemax = -1
      else
         read(cline,*,err=105) wusemin,wusemax
      end if

c     Now we know what x and y are for each image and we have to figure
c     out how to lay the plots out on the page.
      xmin = 1e6
      xmax = 0
      ymin = 1e6
      ymax = 0
      wmin = 1e9
      wmax = -1e9
      fmin = 1e9
      fmax = -1e9
      do i=1,nfile
C     Minimum and maximum X_fine and Y_fine values.
         xmin = min(xmin,xfine(i))
         xmax = max(xmax,xfine(i))
         ymin = min(ymin,yfine(i))
         ymax = max(ymax,yfine(i))
C     Minimum and maximum wavelength and flux values.
         fmaxtmp=0
         do j=1,nz
            wmin = min(wmin,wave(i,j))
            wmax = max(wmax,wave(i,j))
            fmin = min(fmin,flux(i,j))
            fmax = max(fmax,flux(i,j))
         end do
      end do
C     Use smaller wavelength range if specified.
      if (wusemin.ne.-1.AND.wusemax.ne.-1) then
         wmin=wusemin
         wmax=wusemax
      end if
C     Copy of X and Y_fine arrays.
      xcopy(:) = xfine(:)
      ycopy(:) = yfine(:)
C     Initialize unused X and Y_fine array elements to zero for sorting
C     procedure.
      do i=nfile+1,NMAXIM
         xcopy(i)=0
         ycopy(i)=0
      end do
C     Sort X and Y, and find unique values.  Number of panels in plot
c     will be [nxuniq*nyuniq] (apparently zeros are ignored in unique
c     subroutine).
      call sort(nfile,xcopy)
      call sort(nfile,ycopy)
      call unique(nfile,xcopy,nxuniq,xuniq)
      call unique(nfile,ycopy,nyuniq,yuniq)

C==========
C Plotting
C==========

C     Has postscript been produced?
      dops = 0
C     Set up screen device, with requisite number of panels.  Need to
c     specify maximum values for # of panels on each axis, otherwise
c     plot could be unreadable.
 110  if (dops.eq.0) then
         pgstat = pgbeg(0,'/XS',1,1)
         if (pgstat .ne. 1) then
            write(*,*) "/XS device unavailable; trying /AQT."
            pgstat = pgbeg(0,'/AQT',1,1)
            if (pgstat .ne. 1) then
               write(*,*) "/AQT device unavailable; querying."
               pgstat = pgbeg(0,'?',1,1)
            end if
         end if
      else
         write(psfile,'(a,i4.4,a,i4.4,a)') "ringplot_",nimstart,"to"
     $        ,nimend,".ps"
         pgstat = pgbeg(0,psfile//'/PS',1,1)
         if (pgstat.ne.1) then
            write(*,*)
     $           "/PS device unavailable; not producing postscript."
            go to 200
         else
            write(*,'(a,a30,/)') "Writing output plot: ",psfile
         end if
      end if
c      call pgeras()
      call pgscf(1)
      call pgslw(2)
      call pgsch(1.25)
C     Turn on new page prompting
      call pgask (.true.)
c     Loop through the possible X and Y values for plotting.
C     Skip zero values in X and Y arrays.
      if (dofit.eq.1.AND.dops.eq.0) then
         write(*,'(a)') "Results of Voigt profile fits:"
         write(*,'(5(a8))') "File","X_fine" ,"Y_fine","Peak","Width"
      end if
      dvpx=0.9/nxuniq
      dvpy=0.9/nyuniq
      vpxh=0.05
      firstplot=0
 120  do i=1,nxuniq
         vpyh=0.05
         vpxl=vpxh
         vpxh=vpxl+dvpx
         do j=1,nyuniq
            vpyl=vpyh
            vpyh=vpyl+dvpy
            dupli=0
            do k=1,nfile
               if (xfine(k).eq.xuniq(i) .AND. yfine(k).eq.yuniq(j)) then
                  if (dupli .eq. 0) then
                     wavetmp(:)=wave(k,1:nz)
                     fluxtmp(:)=flux(k,1:nz)
                     ferrtmp(:)=ferr(k,1:nz)
                     rnametmp=rnamearr(k)
                     if (doshift.eq.1) then
                        if (firstplot.eq.0) then
                           wavefmaxref=wavefmax(k)
                        else
                           wavefmaxdiff=wavefmax(k)-wavefmaxref
                           do l=1,nz
                              wavetmp(l)=wavetmp(l)-wavefmaxdiff
                           end do
                        end if
                     end if
c     Set up window
                     if (nfile.eq.1) then
                        call pgsvp(0.05,0.95,0.05,0.90)
                     else
                        call pgsvp(vpxl,vpxh,vpyl,vpyh)
                     end if
                     call pgswin(wmin,wmax,fmin,fmax)
                     call pgbox('BCST',0,0,'BCST',0,0)
                     if (i.eq.nxuniq.AND.j.eq.nyuniq) then
                        call pgsch(0.85)
                        call pgswin(wmin,wmax,fmin,fmax)
                        call pgbox('BCMST',0,0,'BCMST',0,0)
                        call pgswin(wmin,wmax,fmin,fmax)
                        call pgsch(1.25)
                     end if
c     Plot data
                     call pgline(nz,wavetmp,fluxtmp)
C     Plot line through peak
                     call pgsls(2)
                     plotpkf = (/ 0, 1000000000 /)
                     if (doshift.eq.1) then
                        plotpkw = (/ wavefmaxref, wavefmaxref /)
                     else
                        plotpkw = (/ wavefmax(k), wavefmax(k) /)
                     end if
                     call pgline (2, plotpkw, plotpkf)
                     call pgsls(1)
C     Voigt profile fit
                     if (dofit.eq.1) then
                        fitguess(1)=0
                        if (doshift.eq.1) then
                           wfmin = wavefmaxref-vpwidth/2
                           wfmax = wavefmaxref+vpwidth/2
                           fitguess(2) = wavefmaxref
                        else
                           wfmin = wavefmax(k)-vpwidth/2
                           wfmax = wavefmax(k)+vpwidth/2
                           fitguess(2) = wavefmax(k)
                        end if
                        npfit = 0
                        do l=1,nz
                           if (wavetmp(l) .gt. wfmin .and. wavetmp(l)
     $                          .lt.wfmax)then
                              npfit = npfit+1
                              wavefit(npfit) = wavetmp(l)
                              fluxfit(npfit) = fluxtmp(l)
                              ferrfit(npfit) = ferrtmp(l)
                           end if
                        end do
                        call evoigt (wavefit,fluxfit,ferrfit,npfit,sgint
     $                       ,slint,iord,vgtfit,fiterr,chisq,flag
     $                       ,fitguess)
                        do l=1,npfit
                           call voigt(wavefit(l),vgtfit,sgint,vplot(l)
     $                          ,dv)
                        end do
                        cont(k) = vgtfit(1)
                        inty(k) = vgtfit(2)
                        cwl(k) = vgtfit(3)
                        sg(k) = vgtfit(4)
                        sl(k) = vgtfit(5)
                        sigsum(k) = sqrt (sg(k)**2 + sl(k)**2)
                        if (dops.eq.0) write (*,'(a8,2(f8.0),2(f8.2))')
     $                       rnametmp(1:7),xfine(k),yfine(k),cwl(k)
     $                       ,sigsum(k)
c     Plot Voigt fit
                        call pgqci(indexc)
                        call pgsci(2)
                        call pgline(npfit,wavefit,vplot)
                        call pgsci(indexc)
                        write(siglabel,'(f6.2)') sigsum(k)
                        call pgsch(0.85)
                        call pgmtxt('T',-1.5,0.95,1.,siglabel)
                        call pgsch(1.25)
                     end if
c     Plot Labels
                     if (i.eq.1.AND.j.eq.1) then
                        write(xlab,'(a4,f5.0)') "X = ",xuniq(i)
                        write(ylab,'(a4,f5.0)') "Y = ",yuniq(j)
                        call pgmtxt('B',1.,0.5,0.5,xlab)
                        call pgmtxt('L',0.5,0.5,0.5,ylab)
                     else if (j.eq.1) then
                        write(xlab,'(f5.0)') xuniq(i)
                        call pgmtxt('B',1.,0.5,0.5,xlab)
                     else if (i.eq.1) then 
                        write(ylab,'(f5.0)') yuniq(j)
                        call pgmtxt('L',0.5,0.5,0.5,ylab)
                     end if
                     call pgsch(0.85)
                     if (firstplot.eq.0) then
                        write(toplabel,'(a7)') rnametmp(1:7)
                        call pgmtxt('T',-1.5,0.05,0.,toplabel)
                     else
                        write(toplabel,'(a4)') rnametmp(4:7)
                        call pgmtxt('T',-1.5,0.05,0.,toplabel)
                     end if
                     call pgsch(1.25)
                     dupli=1
                     firstplot=1
                  else
                     if (dops.eq.0) write(*,'(a,f5.0,a,f5.0,a)')
     $                    "Duplicate spectrum for [X,Y]_fine = ["
     $                    ,xfine(k),",",yfine(k)
     $                    ,"]; plotting first spectrum only."
                  end if
               else if (i.eq.1.OR.j.eq.1) then
                  call pgsvp(vpxl,vpxh,vpyl,vpyh)
                  if (i.eq.1.AND.j.eq.1) then
                     write(xlab,'(a4,f5.0)') "X = ",xuniq(i)
                     write(ylab,'(a4,f5.0)') "Y = ",yuniq(j)
                     call pgmtxt('B',1.,0.5,0.5,xlab)
                     call pgmtxt('L',0.5,0.5,0.5,ylab)
                  else if (j.eq.1) then
                     write(xlab,'(f5.0)') xuniq(i)
                     call pgmtxt('B',1.,0.5,0.5,xlab)
                  else if (i.eq.1) then 
                     write(ylab,'(f5.0)') yuniq(j)
                     call pgmtxt('L',0.5,0.5,0.5,ylab)
                  end if
               end if
            end do
         end do
      end do
      write(toplabel1,'(a,a11,a9)') "UT ",date(1),time(1)
      write(toplabel2,'(a,i2,a,a20,a,f5.0)') "Z_c=",zcoarse(1)
     $     ,"  Filter: ",filter(1),"Z_f=",zfine(1)
      call pgsvp(0.05,0.95,0.05,0.95)
      call pgsch(0.85)
      call pgmtxt('T',1.7,0.02,0.,toplabel1)
      call pgmtxt('T',0.5,0.02,0.,toplabel2)
      call pgsch(1.25)

C     Produce postscript output?
      if (dops.eq.0) then
         dops=1
         go to 110
      end if
      
      call pgend()

 200  end
